import json
from datetime import datetime
from dataEngine import Common, Aws, Channel, AppConfig, Logger

logger = Logger.get()


class Event(Common.PbxDataObject):
    def __init__(self, ID, status, caller, destination):
        # print("====Init event " + str(ID))
        self.ID = ID
        self.name = None
        self.status = status
        self.service = None
        self.app = None
        self.destination = destination
        self.trunk = None
        self.queue = None
        self.page = None
        self.tags = None
        self.season = None
        self.transferType = None
        self.recording = None
        self.mergeWith = None
        self.cause = None
        self.causeStr = None
        self.who = None
        self.time = datetime.now()
        self._caller = caller
        self._callee = {}
        self._flags = []
        self._changed = False

    def set(self, param, value):
        self._changed = (value != getattr(self, param))
        if value:
            setattr(self, param, value)

    @property
    def caller(self):
        return self._caller.get_info() if self._caller else {}

    @caller.setter
    def caller(self, new_val):
        self._caller = new_val

    @property
    def callee(self):
        return self._callee.get_info() if self._callee else {}

    @callee.setter
    def callee(self, new_val):
        self._callee = new_val

    @property
    def flags(self):
        return self._flags

    @flags.setter
    def flags(self, new_val):
        self._changed = (new_val != self._flags)
        if new_val is None:
            self._flags = []
        else:
            self._flags = new_val

    def isChanged(self):
        return self._changed

    def _get_data(self, attrs):
        rv = {}
        for attr in attrs:
            value = self.get(attr)
            if value:
                rv[attr] = value
                setattr(self, attr, None)
        return rv

    # START/UPDATE
    def get_data(self):
        attrs = ["status", "caller", "callee", "service", "destination", "trunk", "queue", "tags", "flags", "page"]
        return self._get_data(attrs)

    def get_data_split(self):
        attrs = ["reason", "service", "transferType", "mergeWith", "status", "caller", "callee", "destination",
                 "trunk", "queue", "tags", "flags"]
        return self._get_data(attrs)

    def get_data_agent(self):
        return {}

    def get_data_record_stop(self):
        rv = {}
        if self.recording is not None:
            rv["fileName"] = self.recording
            self.recording = None
        return rv

    def get_data_end(self):
        attrs = ["cause","causeStr","who"]
        return self._get_data(attrs)

    def push(self, name):
        # print("\n\n>>> Event.push, self: {}\n\n".format(self))
        data = None
        self.time = datetime.now()
        if name in ("CALL.START", "CALL.UPDATE"):
            data = self.get_data()
        elif name == "CALL.SPLIT":
            data = self.get_data_split()
        elif name == "CALL.RECORD_STOP":
            data = self.get_data_record_stop()
        elif name == "CALL.END":
            data = self.get_data_end()

        push(name, self.ID, data)


def push(name, id, data):
    ev = {
        "ext" if name == "PRESENCE" else "id": id,
        "type": name,
        "time": int(datetime.timestamp(datetime.now())*1000),
        "pbx": Common.PBX_SERIAL
    }
    if data:
        ev["data"] = data

    payload = json.dumps(ev, cls=Common.PbxDataEncoder)
    if AppConfig.get("out_file"):
        with open(AppConfig.get("out_file"), "a") as log_file:
            log_file.write("{}\n".format(payload))
    print("\n\033[92m{},\033[0m".format(payload))
    if not AppConfig.isPushDisabled():
        payload_bytes = payload.encode('utf-8')
        put_response = Aws.KINESIS_CLIENT.put_record(
            StreamName=Aws.AWS_STREAM,
            Data=payload_bytes,
            PartitionKey=Common.PBX_SERIAL)
    # print("Response: " + str(put_response))
