import time
import os
from scapy.all import rdpcap
from threading import Thread
from dataEngine import AppConfig, AmiMock, Ldap, Logger

logger = Logger.get()


def process_pcap():
    while not AmiMock.get_loogged_state():
        if PCAP_READER.exit:
            break
        time.sleep(0.5)
    for file in AppConfig.get("in_file"):
        if not os.path.isfile(file):
            logger.warning("File {} not found".format(file))
            continue
        if PCAP_READER.exit:
            break
        ldap_file = file.replace(".pcap", ".ldap")
        if os.path.isfile(ldap_file):
            Ldap.read(ldap_file)
            AppConfig.set("have_ldap_storage", True)
        else:
            AppConfig.set("have_ldap_storage", False)
        logger.debug("Start to read PCAP file: {}".format(file))
        if AppConfig.get("out_file"):
            with open(AppConfig.get("out_file"), "a") as log_file:
                log_file.write("===== Start to read file: {} =====\n".format(file))
        scapy_cap = rdpcap(file)
        timestamp = 0
        for packet in scapy_cap:
            if PCAP_READER.exit:
                break
            p = packet.sprintf("{IP:{TCP:%TCP.sport%}}")
            if not p.isdigit():
                continue
            srcport = int(p)
            if srcport != 5039:
                continue
            payload = packet.sprintf("{IP:{TCP:%Raw.load%}}").replace("'", "").replace("\\r", "\r").replace("\\n", "\n").replace("\\\\", "\\")
            if payload.startswith("Event"):
                if AppConfig.get("realtime"):
                    if timestamp > 0:
                        sleep = packet.time - timestamp
                        time.sleep(sleep)
                AmiMock.send(payload)
                timestamp = packet.time
    if not AppConfig.isStopped():
        AppConfig.set("stop", True)


def init():
    if AppConfig.isTest():
        PCAP_READER.thread.start()


def close():
    PCAP_READER.exit = True
    if PCAP_READER.thread.is_alive():
        PCAP_READER.thread.join()


class PcapReader():
    def __init__(self):
        self.thread = Thread(target=process_pcap)
        self.exit = False


PCAP_READER = PcapReader()
